# Copyright (c) 2001-2005, RedIRIS. All Rights Reserved.
#
# You may distribute under the terms of the GNU General Public License,
# as specified in the LICENSE file that was shipped with this distribution.

package PAPI::ApachePoA;

use strict;
use vars qw/ @ISA /;
use PAPI::PoA;
use Fcntl;
use MLDBM::Sync;
use MLDBM qw(DB_File);
use MLDBM qw(MLDBM::Sync::SDBM_File);
use Data::Dumper; 
use Apache2::Const qw(:common);
use Apache2::Module ();
use Apache2::Log ();
use Apache2::RequestRec ();
@ISA = qw/ PAPI::PoA /;

sub new
{
	my $classname = shift;
	my $self = $classname->SUPER::new(@_);
	return bless $self, $classname;
}

sub set_parameters
{
	my $self=shift;
	my $r=shift;
	
	$self->{'ApacheRequest'} = $r;
	$self->{'Domain'}=$r->get_server_name if ($self->{'Domain'} eq "");

	my $serv_id = $r->dir_config('Service_ID');
	if ( !defined($serv_id) || $serv_id eq ''){
	   $self->error("Can't instance a new PoA, the Service_ID variable is not defined or is empty.");
	   return 0;
	}

		
	$self->SUPER::set_parameters($self->{'Domain'}, $serv_id);

	if ($self->{'Type'} eq "Location") {
		$self->{'Location'}=$self->{'String'};
		$self->{Hcook_Name}="PAPIHcook_L_" . $self->{Serv};
		$self->{Lcook_Name}="PAPILcook_L_" . $self->{Serv};
	}
	elsif ($self->{'Type'} eq "Directory") {
		$self->{'Location'}="/";
		$self->{Hcook_Name}="PAPIHcook_D_" . $self->{Serv};
		$self->{Lcook_Name}="PAPILcook_D_" . $self->{Serv};
	}
	elsif ($self->{'Type'} eq "Files") {
		$self->{'Location'}="/";
		$self->{Hcook_Name}="PAPIHcook_F_" . $self->{Serv};
		$self->{Lcook_Name}="PAPILcook_F_" . $self->{Serv};
	}
	$self->{'Loc'} = $self->{'Location'};
	$r->pnotes('PAPIid' => $self->{'Id'});  

	$self->{'log'} = $self->{'ApacheRequest'}->log();
	return 1;
}

sub error
{
	my $self=shift;
	my $str=shift;
	my $log=$self->{'ApacheRequest'}->log();

	$log->error("PAPI#$self->{'Id'}: $str");
}

sub warning
{
	my $self=shift;
	my $str=shift;
	my $log=$self->{'ApacheRequest'}->log();

	$log->warn("PAPI#$self->{'Id'}: $str");
}

sub accesslog
{
	my $self=shift;
	my $str=shift;
	my $r=$self->{'ApacheRequest'};
	my $log=$r->log();

	$log->notice("PAPI#$self->{'Id'}: $str");
	my $on = $r->pnotes('PAPI');
	if ($on eq "") { $r->pnotes('PAPI' => "PAPI: [$str]"); }
        else { $r->pnotes('PAPI' => $on . " [$str]"); }
}

sub debug
{
	my $self=shift;
	my $str=shift;
	my $log=$self->{'ApacheRequest'}->log();
	
$log->debug("PAPI-DEBUG#$self->{'Id'}: $str");
} 

sub get_cookies
{
	my $self=shift;
	my $r=$self->{'ApacheRequest'};

	return $self->SUPER::get_cookies($r->headers_in->{Cookie} || '');
}

sub gen_Hcook
{
	my $self=shift;
	my $code=shift;
	my $valid_date=shift;
	my $orig_date=shift;
	my $temporal=shift;
	my $crc=shift;
	my $key=$code;
	return undef unless $self->{useHcook};
	if (defined $self->{AddressInToken} && $self->{AddressInToken} eq "1") {
		my $ip=$self->{'ApacheRequest'}->connection()->remote_ip();
		$code.="%".$ip."%";
	}
	if(defined $crc){
		return $self->SUPER::gen_Hcook($code,$valid_date,$orig_date,$temporal,$key,$crc);
	}
	else{
		return $self->SUPER::gen_Hcook($code,$valid_date,$orig_date,$temporal,$key);
	}
}

sub test_Hcook
{
	my $self=shift;
	my $Hcook=shift;
	my $gen_crc=shift;
	my $reqas=shift;

    if (!defined $Hcook) {
    	$self->error("test_Hcook: Unable to test undef value. Is UseHcook set?");
		return (0,0,1,0,0);
    }
	my ($code,$valid_date,$temporal,$new_crc,$gpoa_vdate)=$self->SUPER::test_Hcook($Hcook,$gen_crc,$reqas);
        if ($valid_date<=0) {
		return ($code,$valid_date,$temporal,$new_crc,$gpoa_vdate);
	}
	if (defined $self->{AddressInToken} && $self->{AddressInToken} eq "1") {
		my ($rcode,$cip,$d2)=split /%/,$code;
		my $ip=$self->{'ApacheRequest'}->connection()->remote_ip();
		if ($ip ne $cip) {
			$self->warning("**COLLISION**: Hcook received from address $ip when it was assigned to $cip");
			return (0,-1,1,0,0);
		}
		return ($rcode,$valid_date,$temporal,$new_crc,$gpoa_vdate);
	}
	return ($code,$valid_date,$temporal,$new_crc,$gpoa_vdate);
}

sub gen_Lcook
{
	my $self=shift;
	my $code=shift;
	my $valid_date=shift;
	my $orig_date=shift;
	my $key=$code;
	if (defined $self->{AddressInToken} && $self->{AddressInToken} eq "1") {
		my $ip=$self->{'ApacheRequest'}->connection()->remote_ip();
		$code.="%".$ip."%";
	}
	return $self->SUPER::gen_Lcook($code,$valid_date,$orig_date);
}

sub test_Lcook
{
	my $self=shift;
	my $Lcook=shift;
	my $reqas=shift;
	my ($rt,$valid_date,$gpoa_vdate,$code)=$self->SUPER::test_Lcook($Lcook,$reqas);
        if ($rt==0) { return ($rt,$valid_date,$gpoa_vdate,$code); }
	if (defined $self->{AddressInToken} && $self->{AddressInToken} eq "1") {
		my ($rcode,$cip,$d2)=split /%/,$code;
		my $ip=$self->{'ApacheRequest'}->connection()->remote_ip();
		if ($ip ne $cip) {
			$self->warning("**COLLISION**: Lcook received from address $ip when it was assigned to $cip");
			return (0,$valid_date,$gpoa_vdate,$code);
		}
		return ($rt,$valid_date,$gpoa_vdate,$rcode);
	}
	return ($rt,$valid_date,$gpoa_vdate,$code);
}

sub load_request
{
        my $self=shift;
        my $id=shift;
 
        my($base, %db);
 
        if(!($base = tie(%db, 'MLDBM::Sync',$self->{'Req_DB'}, O_CREAT|O_RDWR, 0644))) {
                $self->error("Opening request DB for reading: $!");
                return undef;
        }
 
        my $req;
 
        $req=$db{$id};
        delete $db{$id};
 
        $self->debug("Loading key=#$id# Req" . Dumper($req)) if ($self->{'Debug'});
        untie %db;
        undef $base;
        return $req;
}
 
sub save_request
{
	my $self=shift;
	my $id=shift;
	my $r=$self->{'ApacheRequest'};

	my($base, %db);

	if(!($base = tie(%db, 'MLDBM::Sync',$self->{'Req_DB'}, O_CREAT|O_RDWR, 0644))){
                $self->error("Opening request DB for writing: $!");
                return undef;
	}
 
	if (defined $db{$id}){
                delete $db{$id};
	}
	my %req;
	$req{headers}=$r->headers_in;
	$req{method}=$r->method;
	$req{uri}=$r->uri;
	$req{args}=$r->args;
	$req{filename}=$r->filename;

	if ($req{method} eq 'POST'){
		my $len = $r->headers_in->{'Content-length'};
		my $buf;
		$r->read($buf,$len);
		$req{content}=$buf;
	}
	$self->debug("Saving key=#$id# req:" . Dumper(\%req)) if ($self->{'Debug'});

	$db{$id}=\%req;

	untie %db;
	undef $base;
}

# The stringArgsToHash subroutine builds a hash from a args string
#
sub stringArgsToHash
{
    my $self = shift;
    my $argsString = shift;
    my %outHash = ();
    
    my @argshash = split ("&",$argsString);
    foreach my $largs (@argshash){
        my($lkey, $value) = split ("=", $largs);
        $outHash{$lkey}=$value;
    }
    
    return %outHash;
}

sub dumpApachePoA {
    my $self = shift;
    my $fileName = shift;
    
    if ( $fileName eq '' ){
    	$fileName = "PoADump.";
    }
    
    my $pintame = pintaHash($self);
    my $fileName = "/tmp/".$fileName.$self->{'Serv'};
    unlink ($fileName);
    open (FOUT, ">$fileName");
    print FOUT $pintame;
    close (FOUT);
}

sub pintaHash {
    my $hash = shift;
    my $salida = '';
    
    foreach my $aux (sort (keys  %{$hash})){
        if ( ref($hash->{$aux}) eq "SCALAR") {
            $salida .= "$aux => $hash->{$aux}\n";
        }elsif ( ref($hash->{$aux}) eq "ARRAY" ){
            $salida .= "Start of array ";
            $salida .= "<$aux>";            
            $salida .= pintaArray($hash->{$aux});
            $salida .= "End of array\n";
        }elsif ( ref($hash->{$aux}) eq "HASH" ){
            $salida .= "Start of hash ";
            $salida .= "<$aux>\n";
            $salida .= pintaHash($hash->{$aux});
            $salida .= "End of hash\n";
        }else{
            $salida .= "<$aux>  $hash->{$aux}\n";
        }
    }
    
    return $salida;
}

sub pintaArray {
    my $vector = shift;
    my $salida = '';
    
    if (ref ($vector) eq "ARRAY"){
        my $tam = @{ $vector };
        $salida .= " of size: $tam\n";
    }else {
        $salida .= "The input parameter is not an array\n";
        return $salida;
    }
    
    foreach my $aux ( @{ $vector }){
        if ( ref($aux) eq "SCALAR" ){
            $salida .= "$aux\n";
        }elsif ( ref($aux) eq "ARRAY" ){
            $salida .= "Start of array ";
            $salida .= pintaArray($aux);
            $salida .= "End of array\n";
        }elsif ( ref($aux) eq "HASH" ){
            $salida .= "Start of hash\n";
            $salida .= pintaHash($aux);
            $salida .= "End of hash\n";
        }else{
            $salida .= "<$aux>\n";
        }
    }
    
    return $salida;
}
#
## End of changes
#
1;

__END__
